import json
import os
import io
import requests
import zipfile
from google.oauth2 import service_account
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload

import toml
from pathlib3x import Path
from rich.console import Console
from worker_automate_hub.config.settings import get_package_version, load_env_config
from worker_automate_hub.utils.logger import logger

console = Console()


def write_env_config(env_dict: dict, google_credentials: dict):
    try:
        print("env: ", env_dict)
        home_dir = Path(os.path.expanduser("~"))
        config_path = home_dir / "worker-automate-hub"
        config_path.mkdir(exist_ok=True)        
        assets_path = config_path / "assets"
        logs_path = config_path / "logs"
        assets_path.mkdir(exist_ok=True)
        logs_path.mkdir(exist_ok=True)

        config_file_path = config_path / "settings.toml"
        config_data = {
            "name": "WORKER",
            "params": {
                "api_base_url": env_dict["API_BASE_URL"],
                "api_auth": env_dict["API_AUTHORIZATION"],
                "notify_alive_interval": env_dict["NOTIFY_ALIVE_INTERVAL"],
                "version": get_package_version("worker-automate-hub"),
                "log_level": env_dict["LOG_LEVEL"],
                "drive_url": env_dict["DRIVE_URL"]
            },
            "google_credentials": google_credentials["content"]
        }
        
        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)
            console.print(f"\nArquivo de configuração criado em: {config_file_path}\n", style="green")

        logger.info(f"Arquivo de configuração do ambiente criado em {config_path}")
        return {
            "Message": f"Arquivo de configuração do ambiente criado em {config_path}",
            "Status": True,
        }
    except Exception as e:
        logger.error(f"Erro ao criar o arquivo de configuração do ambiente. Comando retornou: {e}")
        return {
            "Message": f"Erro ao criar o arquivo de configuração do ambiente.\n Comando retornou: {e}",
            "Status": False,
        }


def add_worker_config(worker):
    try:
        home_dir = Path(os.path.expanduser("~"))
        config_file_path = home_dir / "worker-automate-hub" / "settings.toml"

        if not config_file_path.exists():
            raise FileNotFoundError(f"O arquivo de configuração não foi encontrado em: {config_file_path}")

        with open(config_file_path, "r") as config_file:
            config_data = toml.load(config_file)

        config_data["id"] = {
            "worker_uuid": worker["uuidRobo"],
            "worker_name": worker["nomRobo"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        console.print(f"\nInformações do worker adicionadas ao arquivo de configuração em {config_file_path}\n", style="green")
        console.print("\nConfiguração finalizada com sucesso!\n", style="bold green")
        return {
            "Message": f"Informações do worker adicionadas ao arquivo de configuração em {config_file_path}",
            "Status": True,
        }
    except Exception as e:
        return {
            "Message": f"Erro ao adicionar informações do worker ao arquivo de configuração.\n Comando retornou: {e}",
            "Status": False,
        }

def list_files_in_folder(folder_id, service):
    query = f"'{folder_id}' in parents"
    results = service.files().list(q=query, pageSize=1000, fields="files(id, name, mimeType)").execute()
    items = results.get('files', [])
    print("items: ", items)
    return items

def download_file(file_id, file_name, output_folder, service):
    request = service.files().get_media(fileId=file_id)
    file_path = os.path.join(output_folder, file_name)
    
    with io.FileIO(file_path, 'wb') as fh:
        downloader = MediaIoBaseDownload(fh, request)
        done = False
        while done is False:
            status, done = downloader.next_chunk()
            print(f"Downloading {file_name}, {int(status.progress() * 100)}% complete.")

def download_assets_and_extract_from_drive():
    try:
        console.print("\nIniciando download dos assets...\n", style="bold green")
        env_config, creds_loaded = load_env_config()
        creds_loaded = json.loads(creds_loaded)["web"]   
        folder_url = env_config["DRIVE_URL"]
        home_dir = Path.home()
        config_path = home_dir / "worker-automate-hub"
        config_path.mkdir(exist_ok=True)               
        output_folder = config_path / "assets"
        SCOPES = ['https://www.googleapis.com/auth/drive.readonly']

        credentials = service_account.Credentials.from_service_account_file(
        f"{config_path}\\credentials.json", scopes=SCOPES)
        service = build('drive', 'v3', credentials=credentials)
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)
        
        folder_id = folder_url.split('/')[-1].replace("?usp=drive_link", "")
        files = list_files_in_folder(folder_id, service)
        
        for file in files:
            file_id = file['id']
            file_name = file['name']
            
            download_file(file_id, file_name, output_folder, service)
            
            file_path = os.path.join(output_folder, file_name)
            if zipfile.is_zipfile(file_path):
                with zipfile.ZipFile(file_path, 'r') as zip_ref:
                    zip_ref.extractall(output_folder)
                os.remove(file_path)  # Remove the zip file after extraction

        console.print("\nAssets baixados com sucesso!\n", style="bold green")
    except Exception as e:
        logger.error(f"Erro ao baixar os assets: {e}")
        console.print(f"\nErro ao baixar os assets: {e} \n", style="bold red")
        


import asyncio
import warnings
import time
from datetime import datetime
import pyperclip
import re

import pyautogui
from rich.console import Console
from pywinauto.application import Application

from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    kill_process,
    type_text_into_field,
    api_simplifica,
    find_element_center
)

from worker_automate_hub.utils.util import login_emsys

console = Console()

ASSETS_BASE_PATH = 'assets/descartes_images/'

async def descartes(task):
    #TODO depois tem q tirar o user do emsys q ta hard
    #prod
    # task["configEntrada"]["user"] = "rpa.woody"
    # task["configEntrada"]["pass"] = "rpa321"
    # dev
    task["configEntrada"]["user"] = "RPA.MARVIN"
    task["configEntrada"]["pass"] = "cba321"

    try:
        console.print(task)
        itens = task['configEntrada']['itens']

        #Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(task, app)
        
        if return_login['sucesso'] == True:
            type_text_into_field('Cadastro Pré Venda', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            time.sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Cadastro Pre Venda' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login

        time.sleep(7)

        #Pesquisa "cadastro pre-venda" só que no pyautogui
        # search_menu = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "search_menu.png", confidence=0.8, grayscale=True)
        # if search_menu is not None:
        #     print("Achou:"+ str(search_menu))
        #     pyautogui.click(pyautogui.center(search_menu))
        #     pyautogui.write('Cadastro Pre Venda')
        #     time.sleep(1)
        #     pyautogui.press('enter')
        #     time.sleep(1)
        #     pyautogui.press('enter')
        #     console.print(f"Pesquisa: 'Cadastro Pre Venda' realizada com sucesso", style="bold green")
        # else:
        #     logger.info(f"Error Message: Campo 'Menus de acesso' não encontrado")
        #     console.print(f"Error Message: Campo 'Menus de acesso' não encontrado", style="bold red")


        #Preenche data de validade
        field_validade = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_validade.png", confidence=0.8, grayscale=True)
        if field_validade is not None:
            pyautogui.click(pyautogui.center(field_validade))
            pyautogui.write(f'{datetime.now().strftime("%d/%m/%Y")}', interval=0.1)
            pyautogui.press('tab')
            console.print(f"\nValidade Digitada: '{datetime.now().strftime("%d/%m/%Y")}'", style="bold green")
            time.sleep(1)
        else:
            logger.info(f"\nError Message: Campo 'Validade' não encontrado")
            console.print(f"\nError Message: Campo 'Validade' não encontrado", style="bold red")

        #Condição da Pré-Venda
        condicao_field = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "condicao_pre_venda_descartes.png", confidence=0.8, grayscale=True)
        if condicao_field is not None:
            pyautogui.click(pyautogui.center(condicao_field))
            pyautogui.sleep(1)
            avista = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "a_vista_descartes.png", confidence=0.8, grayscale=True)
            pyautogui.click(pyautogui.center(avista))
            pyautogui.sleep(1)
            console.print(f"\nCondição 'A Vista' Selecionada", style="bold green")
        else:
            logger.info(f"\nError Message: Campo 'Condicao pre-venda' não encontrado")
            console.print(f"\nError Message: Campo 'Condicao pre-venda' não encontrado", style="bold red")

        #Preenche o campo do cliente com o número da filial
        cliente_field = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_cliente.png", confidence=0.8, grayscale=True)
        if cliente_field is not None:
            pyautogui.click(pyautogui.center(cliente_field))
            pyautogui.hotkey("ctrl", "a")
            pyautogui.hotkey("del")
            pyautogui.write(task['configEntrada']['filial'])
            pyautogui.hotkey("tab")
            console.print(f"\nCliente preenchido: '{task['configEntrada']['filial']}'", style="bold green")
            time.sleep(6)
        else:
            logger.info(f"\nError Message: Campo 'Cliente' não encontrado")
            console.print(f"\nError Message: Campo 'Cliente' não encontrado", style="bold red")

        #Clica em cancelar na Janela "Busca Representante"
        try:
            window_busca_representante = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "window_busca_representante.png", confidence=1, grayscale=True)
        except Exception as e:
            window_busca_representante = None 

        if window_busca_representante is not None:
            observacao = f"Posto {task['configEntrada']['filial']} - {task['configEntrada']['nomeEmpresa']} sem representante!"
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
            return {"sucesso": False, "retorno": observacao}
        
        time.sleep(2)

        #Aviso "Deseja alterar a condição de pagamento informada no cadastro do cliente?"
        try:
            payment_condition_warning = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "warning_change_payment_condition.png", confidence=0.95, grayscale=True)
        except:
           payment_condition_warning = None
        if payment_condition_warning is not None:
            button_no = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "warning_button_no.png", confidence=0.8, grayscale=True)
            pyautogui.click(pyautogui.center(button_no))
            console.print(f"\nClicou OK Mensagem 'Deseja alterar a condição de pagamento informada no cadastro do cliente?'", style="bold green")
            time.sleep(6)
        else:
            logger.info(f"\nError Message: Aviso de condição de pagamento não encontrado")
            console.print(f"\nError Message: Aviso de condição de pagamento não encontrado", style="bold red")


        #Seleciona 'Custo Médio' (Seleção do tipo de preço)
        try:
            custo_medio_select = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "select_custo_medio.png", confidence=0.8, grayscale=True)
        except Exception as e:
            custo_medio_select = None
        if custo_medio_select is not None:
            pyautogui.click(pyautogui.center(custo_medio_select))
            button_ok = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "select_ok_custo_medio.png", confidence=0.8, grayscale=True)
            pyautogui.click(pyautogui.center(button_ok))
            time.sleep(5)
            console.print(f"\nClicou OK 'Custo médio'", style="bold green")
        else:
            logger.info(f"\nError Message: Campo 'Custo Médio' não encontrado")
            console.print(f"\nError Message: Campo 'Custo Médio' não encontrado", style="bold red")

        #Clica em ok na mensagem "Existem Pré-Vendas em aberto para este cliente."
        try:
            existing_pre_venda = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "existing_pre_venda.png", confidence=0.8, grayscale=True)
        except:
            existing_pre_venda = None 
        if existing_pre_venda is not None:
            button_ok = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_ok.png", confidence=0.8, grayscale=True)
            pyautogui.click(pyautogui.center(button_ok))
            console.print(f"\nClicou OK 'Pre Venda Existente'", style="bold green")
            time.sleep(5)
        else:
            logger.info(f"\nError Message: Menssagem de prevenda existente não encontrada")
            console.print(f"\nError Message: Menssagem de prevenda existente não encontrada", style="bold yellow")
        
        #Seleciona 'Modelo' BAIXA DE EST.DECORRENTE DE PERDA, ROUBO OU DETERIORAÇÃO
        existing_field_modelo = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_modelo.png", confidence=0.8, grayscale=True)  
        if existing_field_modelo:
            pyautogui.click(existing_field_modelo.left + 100, existing_field_modelo.top)
            pyautogui.click(1700,800)
            pyautogui.write("B")
            time.sleep(2)
            pyautogui.hotkey('tab')
            console.print(f"\nSelecionou 'BAIXA DE EST.DECORRENTE DE PERDA, ROUBO OU DETERIORAÇÃO'", style="bold green")
        #Ultimo passo no menu capa

        #Abre Menu itens
        menu_itens = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "menu_itens.png", confidence=0.99, grayscale=True)
        if menu_itens is not None:
            pyautogui.click(pyautogui.center(menu_itens))
        
        time.sleep(2)

        #Loop de itens
        for item in itens:
            button_incluir = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_incluir.png", confidence=0.9, grayscale=True)
            pyautogui.click(pyautogui.center(button_incluir))
            time.sleep(5)
            console.print("\nClicou em 'Incluir'", style='bold green')            

            # Digita Almoxarifado
            button_almoxarifado = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_almoxarifado.png", confidence=0.99, grayscale=True)
            pyautogui.doubleClick(button_almoxarifado.left + 129, button_almoxarifado.top + 14)
            pyautogui.hotkey('del')
            pyautogui.write(task['configEntrada']['filial']+"99")
            pyautogui.hotkey('tab')
            time.sleep(3)
            console.print(f"\nDigitou almoxarifado {task['configEntrada']['filial']+"99"}", style='bold green')            

            # #Segue para o campo do item
            pyautogui.hotkey('tab')
            pyautogui.hotkey('tab')
            pyautogui.write(item['codigoProduto'])
            pyautogui.hotkey('tab')
            time.sleep(5)
            console.print(f"\nDigitou codigo do produto {item['codigoProduto']}", style='bold green')            

            #Checa tela de pesquisa de item
            try:
                window_pesquisa_item = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "window_pesquisa_item.png", confidence=0.9, grayscale=True)
            except:
                window_pesquisa_item = None
            
            if window_pesquisa_item is not None:
                observacao = f"Item {item['codigoProduto']} não encontrado, verificar cadastro"
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
                return {"sucesso": False, "retorno": observacao}

            #Checa se existe alerta de item sem preço, se existir retorna erro(simplifica e bof)
            try:
                warning_price = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "warning_item_price.png", confidence=0.8, grayscale=False)
            except:
                warning_price = None
            if warning_price is not None:
                observacao = f"Item {item['codigoProduto']} não possui preço, verificar erro de estoque ou de bloqueio."
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
                return {"sucesso": False, "retorno": observacao}
            
            time.sleep(5)
            #Seleciona o Saldo Disponivel e verifica se ah possibilidade do descarte
            field_saldo_disponivel = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_saldo_disponivel.png", confidence=0.8, grayscale=True)
            if field_saldo_disponivel is not None:
                pyautogui.doubleClick(field_saldo_disponivel.left + 20, field_saldo_disponivel.top + 20)
                time.sleep(1)
                pyautogui.doubleClick(field_saldo_disponivel.left + 20, field_saldo_disponivel.top + 20)
                time.sleep(1)
                pyautogui.hotkey('ctrl', 'c')
                amount_avaliable= ''
                amount_avaliable = pyperclip.paste()
                console.print(f"Saldo Disponivel: '{amount_avaliable}'", style="bold green")
                #Verifica se o saldo disponivel é valido para descartar
                if int(amount_avaliable) > 0 and int(amount_avaliable) >= int(item['qtd']): 
                    field_quantidade = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_quantidade.png", confidence=0.8, grayscale=True)
                    pyautogui.doubleClick(pyautogui.center(field_quantidade))
                    pyautogui.hotkey('del')
                    pyautogui.write(str(item['qtd']))
                    pyautogui.hotkey('tab')
                    time.sleep(2)
                else:
                    observacao = f"Saldo Disponivel: '{amount_avaliable}' menor que o valor que deveria ser descartado. Item: '{item['qtd']}'"
                    # if task['configEntrada']['precisaRetorno'] == True:
                    await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
                    console.print(observacao, style="bold red")
                    return {"sucesso": False, "retorno": observacao}

            #Clica em incluir para adicionar o item na nota
            button_incluir_item = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_incluir_item.png", confidence=0.8, grayscale=True)
            if button_incluir_item is not None:
                pyautogui.click(pyautogui.center(button_incluir_item))
                time.sleep(3)

            #Clica em cancelar para fechar a tela e abrir novamente caso houver mais itens
            button_cancela_item = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_cancela_item.png", confidence=0.8, grayscale=False)
            pyautogui.click(pyautogui.center(button_cancela_item))
            time.sleep(3)
        
        time.sleep(2)

        #Clica no botão "+" no canto superior esquerdo para lançar a pre-venda
        button_lanca_pre_venda = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_lanca_prevenda.png", confidence=0.8, grayscale=False)
        pyautogui.click(pyautogui.center(button_lanca_pre_venda))
        console.print("\nLancçou Pré-Venda", style="bold green")
        
        time.sleep(5)
        #Verifica mensagem de "Pré-Venda incluida com número: xxxxx"
        try:
            included_pre_venda = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "included_pre_venda.png", confidence=0.85, grayscale=False)
        except:
            observacao = f'Não achou mensagem pré-venda incluida'
            console.print(observacao,style='bold green')
            return {"sucesso": True, "retorno": observacao}
        
        #Clica no centro da mensagem e copia o texto para pegar o numero da pre-venda
        pyautogui.click(pyautogui.center(included_pre_venda))
        pyautogui.hotkey("ctrl", "c")
        pre_venda_message = ""
        pre_venda_message = pyperclip.paste()
        pre_venda_message = re.findall(r'\d+-\d+', pre_venda_message)
        console.print(f"Numero pré-venda: '{pre_venda_message[0]}'",style='bold green')

        #Clica no ok da mensagem
        button_ok = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_ok2.png", confidence=0.8, grayscale=True)
        pyautogui.click(pyautogui.center(button_ok))
        
        #Clica em Faturar
        button_faturar = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_faturar.png", confidence=0.8, grayscale=False)
        pyautogui.click(pyautogui.center(button_faturar))
        console.print(f"Clicou em: 'Faturar'",style='bold green')
         
        time.sleep(10)

        #Verifica se existe a mensagem de recalcular parcelas
        try:
            message_recalcular = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "message_recalcular_parcelas.png", confidence=0.85, grayscale=False)
        except:
            message_recalcular = None

        #Se existir clica em nao
        if message_recalcular is not None:
            button_no = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "warning_button_no.png", confidence=0.8, grayscale=True)
            pyautogui.click(pyautogui.center(button_no))
        else:
            logger.info(f"Mensagem de para recalcular parcelas da pre-venda nao existe")
            console.print(f"Mensagem de para recalcular parcelas da pre-venda nao existe", style="bold yellow")
        
        time.sleep(8)

        #Seleciona o modelo da nota (em teoria em prod nao precisa mas esta aqui por precaução) -- depreciado
        # pyautogui.hotkey("tab")
        # pyautogui.write("077")
        # time.sleep(5) 

        try:
            model_selected = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_model_select_model_077.png", confidence=0.95, grayscale=True)
            console.print("Modelo já selecionado", style="bold green")
        except:
            console.print("Modelo não está selecionado", style="bold yellow")
            model_selected = None
            
        if model_selected is None:
            console.print("Selecionando Modelo", style="bold green")
            try:
                field_modelo_faturamento = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_modelo_faturamento2.png", confidence=0.9, grayscale=True)
            except:      
                field_modelo_faturamento = None
            
            if field_modelo_faturamento is not None:
                pyautogui.click(field_modelo_faturamento.left, field_modelo_faturamento.top + 50)
                time.sleep(1)
                for _ in range(15):
                    pyautogui.keyDown('up')
                pyautogui.keyUp('up')
                               
            try:
                model_to_select = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "field_model_select_model_077.png", confidence=0.95, grayscale=True)
                pyautogui.click(pyautogui.center(model_to_select))
            except:
                observacao = {
                "Numero Pre Venda": pre_venda_message[0],
                "Numero da nota": ''
                }
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
                return {"sucesso": False, "retorno": "Falha ao selecionar modelo: Na tela de Faturamento Pré-Venda"}

        time.sleep(5)

        #Clicar no botao "OK" com um certo verde
        button_verde = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_ok_verde.png", confidence=0.99, grayscale=True)
        pyautogui.click(pyautogui.center(button_verde))

        time.sleep(5)

        #Aviso "Deseja faturar pré-venda?"
        try:
            faturar_pre_venda = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "faturar_pre_venda.png", confidence=0.9, grayscale=True)
        except:
            faturar_pre_venda = None

        if faturar_pre_venda is not None:
            button_yes = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "warning_button_yes.png", confidence=0.9, grayscale=True)
            pyautogui.click(pyautogui.center(button_yes))
        else:
            observacao = {
            "Numero Pre Venda": pre_venda_message[0],
            "Numero da nota": ''
            }
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
            return {"sucesso": False, "retorno": "Falha ao cliclar em: 'SIM' no aviso: 'Deseja realmente faturar esta Pré-Venda ?'"}

        time.sleep(7)

        #Mensagem de nota fiscal gerada com número
        nota_fical_gerada = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "message_nota_fiscal_gerada.png", confidence=0.99, grayscale=True)
        pyautogui.click(pyautogui.center(nota_fical_gerada))
        pyautogui.hotkey("ctrl", "c")
        nota_fiscal = pyperclip.paste()
        nota_fiscal = re.findall(r'\d+-?\d*', nota_fiscal)
        console.print(f"\nNumero NF: '{nota_fiscal[0]}'",style='bold green')

        time.sleep(7)

        #Transmitir a nota
        transmitir = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_transmitir.png", confidence=0.99, grayscale=True)
        pyautogui.click(pyautogui.center(transmitir))
        logger.error("\nNota Transmitida")
        console.print("\nNota Transmitida", style="bold green")
        
        time.sleep(7)
        #Fechar transmitir nota
        transmitir_fechar = pyautogui.locateOnScreen(ASSETS_BASE_PATH + "button_fechar_transmitir_nota.png", confidence=0.99, grayscale=True)
        pyautogui.click(pyautogui.center(transmitir_fechar))

        observacao = {
            "Numero Pre Venda": pre_venda_message[0],
            "Numero da nota": nota_fiscal[0]
        }

        await api_simplifica(task['configEntrada']['urlRetorno'], "SUCESSO", observacao, task['configEntrada']['uuidSimplifica'], nota_fiscal[0])
        return {"sucesso": True, "retorno": observacao}
    
    except Exception as ex:
        observacao = f"Erro Processo Descartes: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], None)
        return {"sucesso": False, "retorno": observacao}


import asyncio
import math
import os
import subprocess
import psutil
import pyautogui
import warnings
import aiohttp

from prompt_toolkit.shortcuts import checkboxlist_dialog, radiolist_dialog
from rich.console import Console
from worker_automate_hub.config.settings import (
    load_env_config,
    load_worker_config,
)
from worker_automate_hub.utils.logger import logger

console = Console()


async def get_system_info():
    worker_config = load_worker_config()
    max_cpu = psutil.cpu_percent(interval=10.0)
    cpu_percent = psutil.cpu_percent(interval=1.0)
    memory_info = psutil.virtual_memory()

    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "maxCpu": f"{max_cpu}",
        "maxMem": f"{memory_info.total / (1024 ** 3):.2f}",
        "usoCpu": f"{cpu_percent}",
        "usoMem": f"{memory_info.used / (1024 ** 3):.2f}",
        "situacao": "{'status': 'em desenvolvimento'}",
    }


async def get_new_task_info():
    env_config, _ = load_env_config()
    worker_config = load_worker_config()
    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "versao": env_config["VERSION"],
    }


async def kill_process(process_name: str):
    try:
        # Obtenha o nome do usuário atual
        current_user = os.getlogin()

        # Liste todos os processos do sistema
        result = await asyncio.create_subprocess_shell(
            f'tasklist /FI "USERNAME eq {current_user}" /FO CSV /NH',
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        stdout, stderr = await result.communicate()

        if result.returncode != 0:
            logger.error(f"Erro ao listar processos: {stderr.decode().strip()}", None)
            console.print(
                f"Erro ao listar processos: {stderr.decode().strip()}", style="bold red"
            )
            return

        if stdout:
            lines = stdout.decode().strip().split("\n")
            for line in lines:
                # Verifique se o processo atual corresponde ao nome do processo
                if process_name in line:
                    try:
                        # O PID(Process ID) é a segunda coluna na saída do tasklist
                        pid = int(line.split(",")[1].strip('"'))
                        await asyncio.create_subprocess_exec(
                            "taskkill", "/PID", str(pid), "/F"
                        )
                        # logger.info(
                        #     f"Processo {process_name} (PID {pid}) finalizado.", None
                        # )
                        console.print(
                            f"\nProcesso {process_name} (PID {pid}) finalizado.\n",
                            style="bold green",
                        )
                    except Exception as ex:
                        # logger.error(
                        #     f"Erro ao tentar finalizar o processo {process_name} (PID {pid}): {ex}",
                        #     None,
                        # )
                        console.print(
                            f"Erro ao tentar finalizar o processo {process_name} (PID {pid}): {ex}",
                            style="bold red",
                        )
        else:
            logger.info(
                f"Nenhum processo chamado {process_name} encontrado para o usuário {current_user}.",
                None,
            )
            console.print(
                f"Nenhum processo chamado {process_name} encontrado para o usuário {current_user}.",
                style="bold yellow",
            )

    except Exception as e:
        logger.error(f"Erro ao tentar matar o processo: {e}", None)
        console.print(f"Erro ao tentar matar o processo: {e}", style="bold red")


async def find_element_center(image_path, region_to_look, timeout):
    try:
        counter = 0
        confidence_value = 1.00
        grayscale_flag = False

        while counter <= timeout:
            try:
                element_center = pyautogui.locateCenterOnScreen(
                    image_path,
                    region=region_to_look,
                    confidence=confidence_value,
                    grayscale=grayscale_flag,
                )
            except Exception as ex:
                element_center = None
                # logger.info(str(ex), None)
                # console.print(
                #     f"Erro em locateCenterOnScreen: {str(ex)}", style="bold red"
                # )
                console.print(f"Elemento náo encontrado na pos: {region_to_look}")

            if element_center:
                return element_center
            else:
                counter += 1

                if confidence_value > 0.81:
                    confidence_value -= 0.01

                if counter >= math.ceil(timeout / 2):
                    grayscale_flag = True

                await asyncio.sleep(1)

        return None
    except Exception as ex:
        # logger.info(str(ex), None)
        # console.print(f"Erro em find_element_center: {str(ex)}", style="bold red")
        console.print(f"{counter} - Buscando elemento na tela: {region_to_look}", style="bold yellow")
        return None    



def type_text_into_field(text, field, empty_before, chars_to_empty):
    try:
        if empty_before:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)

        field.type_keys(text, with_spaces=True)

        if str(field.texts()[0]) == text:
            return
        else:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)
            field.type_keys(text, with_spaces=True)

    except Exception as ex:
        logger.error("Erro em type_text_into_field: " + str(ex), None)
        console.print(f"Erro em type_text_into_field: {str(ex)}", style="bold red")


async def wait_element_ready_win(element, trys):
    max_trys = 0

    while max_trys < trys:
        try:
            if element.wait("exists", timeout=2):
                await asyncio.sleep(1)
                if element.wait("exists", timeout=2):
                    await asyncio.sleep(1)
                    if element.wait("enabled", timeout=2):
                        element.set_focus()
                        await asyncio.sleep(1)
                        if element.wait("enabled", timeout=1):
                            return True

        except Exception as ex:
            logger.error("wait_element_ready_win -> " + str(ex), None)
            console.print(
                f"Erro em wait_element_ready_win: {str(ex)}", style="bold red"
            )

        max_trys = max_trys + 1

    return False


async def login_emsys(task, app):

    from pywinauto.application import Application
    warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")

    await asyncio.sleep(10)
    # Testa se existe alguma mensagem no Emsys
    window_message_login_emsys = await find_element_center(
        "assets/emsys/window_message_login_emsys.png", (560, 487, 1121, 746), 15
    )

    # Clica no "Não mostrar novamente" se existir
    if window_message_login_emsys:
        pyautogui.click(window_message_login_emsys.x, window_message_login_emsys.y)
        pyautogui.click(
            window_message_login_emsys.x + 383, window_message_login_emsys.y + 29
        )
        console.print("Mensagem de login encontrada e fechada.", style="bold green")

    # Ve se o Emsys esta aberto no login
    image_emsys_login = await find_element_center(
        "assets/emsys/logo_emsys_login.png", (800, 200, 1400, 700), 600
    )

    if image_emsys_login:
        if await wait_element_ready_win(app["Login"]["Edit2"], 80):
            disconect_database = await find_element_center(
                "assets/emsys/disconect_database.png", (1123, 452, 1400, 578), 300
            )

            if disconect_database:
                # Realiza login no Emsys
                type_text_into_field(
                    task["configEntrada"]["user"], app["Login"]["Edit2"], True, "50"
                )
                pyautogui.press("tab")
                type_text_into_field(
                    task["configEntrada"]["pass"],
                    app["Login"]["Edit1"],
                    True,
                    "50",
                )
                pyautogui.press("enter")

                # Seleciona a filial do emsys
                selecao_filial = await find_element_center(
                    "assets/emsys/selecao_filial.png", (480, 590, 820, 740), 350
                )

                if selecao_filial:
                    type_text_into_field(
                        task["configEntrada"]["filial"],
                        app["Seleção de Empresas"]["Edit"],
                        True,
                        "50",
                    )
                    pyautogui.press("enter")

                    button_logout = await find_element_center(
                        "assets/emsys/button_logout.png", (0, 0, 130, 150), 750
                    )

                    if button_logout:
                        console.print(
                            "Login realizado com sucesso.", style="bold green"
                        )
                        return {"sucesso": True, "retorno": "Logou com sucesso no emsys!"}
        else:
            logger.info("Elemento de login não está pronto.")
            console.print("Elemento de login não está pronto.", style="bold red")
            return {"sucesso": False, "retorno": "Falha ao logar no EMSys!"}

async def api_simplifica(urlSimplifica: str, status: str, observacao: str, uuidsimplifica: str, numNota: str):
    
    data = {
        "uuid_simplifica": uuidsimplifica,
        "status": status,
        "numero_nota": numNota,
        "observacao": observacao  
    }

    try:
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{urlSimplifica}",
                data=data,
            ) as response:
                if response == 200:
                    console.print(f"\nSucesso ao enviar {data}\n para o simplifica", style='bold green')
    except Exception as e:
        console.print(f"Erro ao comunicar com endpoint do Simplifica: {e}", style='bold green')
        logger.info(f"Erro ao comunicar com endpoint do Simplifica: {e}")
    
        



import asyncio
import importlib.metadata
import os
import sys

import psutil
from rich.console import Console
from typer import Context, Exit, Option, Typer

from worker_automate_hub.api.client import get_workers, load_environments
from worker_automate_hub.utils.updater import check_for_update
import inquirer
from inquirer.themes import GreenPassion

from worker_automate_hub.core.so_manipulation import add_worker_config, download_assets_and_extract_from_drive, write_env_config
from .worker import run_worker

console = Console()
app = Typer()

HELP_MESSAGE = """
[b]Worker[/] - Grupo Argenta

Forma de uso: [b]worker [SUBCOMANDO] [ARGUMENTOS][/]

Existem 3 subcomandos disponíveis para essa aplicação

- [b]run[/]: Inicializa o Worker na máquina atual e começa a solicitar trabalho para o orquestrador.
- [b]validate[/]: Verifica se o Worker atual está configurado corretamente e pronto para ser inicializado.
- [b]assets[/]: Realiza a limpeza e depois download na pasta assets de todos arquivos utilizado pelo worker durante execução.

[b]Exemplos de uso:[/]
 [b][blue]RUN[/][/]
    [green][b]worker[/][/] [b]run[/]

 [b][blue]UPDATE[/][/]
    [green][b]worker[/][/] [b]update[/]

 [b][blue]VALIDATE[/][/]
    [green][b]worker[/][/] [b]validate[/]

---

[b]Help:[/]
 [b]Para mais informações[/]
    [green][b]worker[/][/] --help

 [b]Para ver a versão instalada[/]
    [green][b]worker[/][/] --version

 [b]Para gerar o arquivo de configuração[/]
    [green][b]worker[/][/] --configure

 [b]Para informações detalhadas
    [blue][link=https://github.com/SIM-Rede/worker-automate-hub]Repo no GIT Argenta[/][/] | [blue][link=https://pypi.org/project/worker-automate-hub/]Publicação no PyPI[/][/]
"""


def function_help(flag: bool):
    if flag:
        console.print(
            importlib.metadata.version("worker-automate-hub"),
            style="bold blue",
        )
        raise Exit(code=0)


def function_configure(flag: bool):

    if flag:
        console.clear()
        environment_names = [            
            "local",
            "qa",
            "main",
        ]
        q =[
            inquirer.Text("vault_token", "Por favor digite o token do Vault"),
            inquirer.List("env_list", "Selecione o ambiente", environment_names)]
        r = inquirer.prompt(q, theme=GreenPassion())

        env_sel, credentials = load_environments(r["env_list"], r["vault_token"])
        write_env_config(env_sel, credentials)
        workers = asyncio.run(get_workers())
        
        if workers == None:
            console.print("\nNenhum worker encontrado.\n", style="yellow")
            raise Exit(code=0)
        else:
        
            nomes_workers = [worker["nomRobo"] for worker in workers]
            q2 = [inquirer.List("worker_list", "Selecione um Worker", choices=nomes_workers)]
            r2 = inquirer.prompt(q2, theme=GreenPassion())
            worker_sel = next(worker for worker in workers if worker["nomRobo"] == r2["worker_list"])
            add_worker_config(worker_sel)
            # download_assets_and_extract_from_drive()

            raise Exit(code=0)


@app.callback(invoke_without_command=True)
def main(
    ctx: Context,
    version: bool = Option(False, callback=function_help, is_flag=True),
    configure: bool = Option(False, callback=function_configure, is_flag=True)    
):
    if ctx.invoked_subcommand:
        return
    console.print(HELP_MESSAGE)

def is_already_running(script_name):
    try:
        for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
            try:
                cmdline = proc.info['cmdline']
                # Verifica se o processo atual é este script, mas não ele próprio
                if proc.info['pid'] != os.getpid() and cmdline and script_name in cmdline:
                    return True
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass
        return False
    except Exception as e:
        console.print(f"\nErro ao verificar se o processo já está em execução: {e}\n", style="bold red")        


@app.command()
def run():
    script_name = sys.argv[0]
    if is_already_running(script_name):
        console.print("O script já está em execução. Saindo...")
        sys.exit(0)
    else:
        asyncio.run(run_worker())


@app.command()
def update():
    check_for_update()



import asyncio
import pyfiglet
from rich.console import Console

from worker_automate_hub.api.client import get_new_task, notify_is_alive
from worker_automate_hub.config.settings import load_env_config, load_worker_config
from worker_automate_hub.tasks.task_executor import perform_task
from worker_automate_hub.utils.logger import logger

console = Console()


async def check_and_execute_tasks():
    while True:
        try:
            task = await get_new_task()
            logger.info(f"Executando a task: {task['data']['nomProcesso']}")
            await perform_task(task["data"])
            await asyncio.sleep(5)
        except Exception as e:
            logger.error(f"Ocorreu um erro de execução: {e}")
            await asyncio.sleep(5)


async def notify_alive():
    env_config, _ = load_env_config()
    while True:
        try:
            logger.info("Notificando last alive...")
            await notify_is_alive()
            await asyncio.sleep(int(env_config["NOTIFY_ALIVE_INTERVAL"]))
        except Exception as e:
            logger.error(f"Erro ao notificar que está ativo: {e}")
            await asyncio.sleep(int(env_config["NOTIFY_ALIVE_INTERVAL"]))


async def main_process():
    env_config, _ = load_env_config()
    worker_config = load_worker_config()
   
    custom_font = "slant"
    ascii_banner = pyfiglet.figlet_format(f"Worker", font=custom_font)
    console.clear()
    console.print(ascii_banner + f" versão: {env_config["VERSION"]}\n", style="bold blue")
    logger.info(f"Worker em execução: {worker_config["NOME_ROBO"]}")
    console.print(f"Worker em execução: {worker_config["NOME_ROBO"]}\n", style="green")  # Mensagem de inicialização

    tasks = [check_and_execute_tasks(), notify_alive()]
    await asyncio.gather(*tasks)


def run_worker():
    try:
        while True:
            asyncio.run(main_process())
            break
    except asyncio.CancelledError:
        console.print("Aplicação encerrada pelo usuário.")



from worker_automate_hub.tasks.jobs.fazer_pudim import fazer_pudim
from worker_automate_hub.tasks.jobs.login_emsys import login_emsys
from worker_automate_hub.tasks.jobs.descartes import descartes

task_definitions = {
    "5b295021-8df7-40a1-a45e-fe7109ae3902": fazer_pudim,
    "a0788650-de48-454f-acbf-3537ead2d8ed": login_emsys,
    "abcfa1ba-d580-465a-aefb-c15ac4514407": descartes
}
